@@ -11,7 +11,6 @@ from signal import SIGINT
 from distutils.version import LooseVersion
 
 import llfuse
-import msgpack
 
 from .logger import create_logger
 logger = create_logger()
@@ -21,6 +20,7 @@ from .archiver import Archiver
 from .archive import Archive
 from .hashindex import FuseVersionsIndex
 from .helpers import daemonize, hardlinkable, signal_handler, format_file_size
+from .helpers import msgpack
 from .item import Item
 from .lrucache import LRUCache
 from .remote import RemoteRepository
@@ -214,6 +214,9 @@ class FuseOperations(llfuse.Operations):
     # mount options
     allow_damaged_files = False
     versions = False
+    uid_forced = None
+    gid_forced = None
+    umask = 0
 
     def __init__(self, key, repository, manifest, args, decrypted_repository):
         super().__init__()
@@ -236,7 +239,7 @@ class FuseOperations(llfuse.Operations):
         self.contents = defaultdict(dict)
         self.default_uid = os.getuid()
         self.default_gid = os.getgid()
-        self.default_dir = Item(mode=0o40755, mtime=int(time.time() * 1e9), uid=self.default_uid, gid=self.default_gid)
+        self.default_dir = None
         self.pending_archives = {}
         self.cache = ItemCache(decrypted_repository)
         data_cache_capacity = int(os.environ.get('BORG_MOUNT_DATA_CACHE_ENTRIES', os.cpu_count() or 1))
@@ -276,19 +279,58 @@ class FuseOperations(llfuse.Operations):
 
     def mount(self, mountpoint, mount_options, foreground=False):
         """Mount filesystem on *mountpoint* with *mount_options*."""
-        options = ['fsname=borgfs', 'ro']
+
+        def pop_option(options, key, present, not_present, wanted_type, int_base=0):
+            assert isinstance(options, list)  # we mutate this
+            for idx, option in enumerate(options):
+                if option == key:
+                    options.pop(idx)
+                    return present
+                if option.startswith(key + '='):
+                    options.pop(idx)
+                    value = option.split('=', 1)[1]
+                    if wanted_type is bool:
+                        v = value.lower()
+                        if v in ('y', 'yes', 'true', '1'):
+                            return True
+                        if v in ('n', 'no', 'false', '0'):
+                            return False
+                        raise ValueError('unsupported value in option: %s' % option)
+                    if wanted_type is int:
+                        try:
+                            return int(value, base=int_base)
+                        except ValueError:
+                            raise ValueError('unsupported value in option: %s' % option) from None
+                    try:
+                        return wanted_type(value)
+                    except ValueError:
+                        raise ValueError('unsupported value in option: %s' % option) from None
+            else:
+                return not_present
+
+        # default_permissions enables permission checking by the kernel. Without
+        # this, any umask (or uid/gid) would not have an effect and this could
+        # cause security issues if used with allow_other mount option.
+        # When not using allow_other or allow_root, access is limited to the
+        # mounting user anyway.
+        options = ['fsname=borgfs', 'ro', 'default_permissions']
         if mount_options:
             options.extend(mount_options.split(','))
-        try:
-            options.remove('allow_damaged_files')
-            self.allow_damaged_files = True
-        except ValueError:
-            pass
-        try:
-            options.remove('versions')
-            self.versions = True
-        except ValueError:
-            pass
+        ignore_permissions = pop_option(options, 'ignore_permissions', True, False, bool)
+        if ignore_permissions:
+            # in case users have a use-case that requires NOT giving "default_permissions",
+            # this is enabled by the custom "ignore_permissions" mount option which just
+            # removes "default_permissions" again:
+            pop_option(options, 'default_permissions', True, False, bool)
+        self.allow_damaged_files = pop_option(options, 'allow_damaged_files', True, False, bool)
+        self.versions = pop_option(options, 'versions', True, False, bool)
+        self.uid_forced = pop_option(options, 'uid', None, None, int)
+        self.gid_forced = pop_option(options, 'gid', None, None, int)
+        self.umask = pop_option(options, 'umask', 0, 0, int, int_base=8)  # umask is octal, e.g. 222 or 0222
+        dir_uid = self.uid_forced if self.uid_forced is not None else self.default_uid
+        dir_gid = self.gid_forced if self.gid_forced is not None else self.default_gid
+        dir_mode = 0o40755 & ~self.umask
+        self.default_dir = Item(mode=dir_mode, mtime=int(time.time() * 1e9), uid=dir_uid, gid=dir_gid)
         self._create_filesystem()
         llfuse.init(self, mountpoint, options)
         if not foreground:
@@ -485,10 +527,10 @@ class FuseOperations(llfuse.Operations):
         entry.generation = 0
         entry.entry_timeout = 300
         entry.attr_timeout = 300
-        entry.st_mode = item.mode
+        entry.st_mode = item.mode & ~self.umask
         entry.st_nlink = item.get('nlink', 1)
-        entry.st_uid = item.uid if item.uid >= 0 else self.default_uid
-        entry.st_gid = item.gid if item.gid >= 0 else self.default_gid
+        entry.st_uid = self.uid_forced if self.uid_forced is not None else item.uid if item.uid >= 0 else self.default_uid
+        entry.st_gid = self.gid_forced if self.gid_forced is not None else item.gid if item.gid >= 0 else self.default_gid
         entry.st_rdev = item.get('rdev', 0)
         entry.st_size = item.get_size()
         entry.st_blksize = 512
@@ -499,10 +541,12 @@ class FuseOperations(llfuse.Operations):
             entry.st_mtime_ns = mtime_ns
             entry.st_atime_ns = item.get('atime', mtime_ns)
             entry.st_ctime_ns = item.get('ctime', mtime_ns)
+            entry.st_birthtime_ns = item.get('birthtime', mtime_ns)
         else:
             entry.st_mtime = mtime_ns / 1e9
             entry.st_atime = item.get('atime', mtime_ns) / 1e9
             entry.st_ctime = item.get('ctime', mtime_ns) / 1e9
+            entry.st_birthtime = item.get('birthtime', mtime_ns) / 1e9
         return entry
 
     def listxattr(self, inode, ctx=None):
